class Actor < GameObject
end

class Result < Actor
  trait :bounding_box
  def setup  
    @image = Image["result.png"]
  end
  
  def invoke
    $window.switch_game_state(End)
  end


end

class Computer < Actor
  attr_reader :anim
  trait :bounding_box
  def setup
    @anim = Animation.new(:file => "computer.bmp", :size => [10,17], :delay => 1, :bounce => true)
    @image = @anim.frames.first
  end
  
  def invoke
    @image = @anim.next
  end  
end

class Teleport < Actor
  trait :bounding_box
  def setup
    @image = Image["teleport.bmp"]
    
  end
  
  def invoke
    if Computer.all.all? { |c| c.anim.index == 1 }
      $window.switch_game_state(Level2)
    end
  end
end

class Scientist < Actor
  trait :bounding_box

  # attributes so editor can read/write
  def text
    options[:text]
  end
  def text=(text)
    options[:text] = text
  end
  
  def setup
    @anim = Animation.new(:file => "scientist.bmp", :size => [4,10], :delay => 300)
    @image = @anim.first
  end
  
  def update
    @image = @anim.next
  end
  
  def invoke
    Talk.destroy_all
    Talk.create(self.text, :x => self.x, :y => self.y - 150, :rotation_center => :center, :max_width => 200, :color => Color::RED, :size => 14, :factor => 2)
  end  
end

class Talk < Text
  trait :timer
  
  def setup
    after(15000) { destroy }
  end
end
