class Level < GameState
  trait :timer
  trait :viewport
  
  def initialize(options = {})
    Song.current_song.stop if Song.current_song
    
    super
    
    @image_name = "#{filename}.png"
    
    #unless defined?(Ocra)
    #  self.input = { :r => :reload, :e => :edit, :holding_a => :left, :holding_s => :down, :holding_d => :right, :holding_w => :up }
    #end
    
    @level = GameObject.create(:image => @image_name, :rotation_center => :top_left)
    viewport.game_area = [0,0,@level.width,@level.height]
    
    load_game_objects(:file => File.join(ROOT, "#{filename}.yml"))
  end
  
  def left; @player.x -= 30;  end
  def right; @player.x += 30; end
  def up; @player.y -= 30;    end
  def down; @player.y += 30;  end
  
  def edit
    push_game_state(GameStates::Edit.new(:grid => false, :except => [Player], :attributes => [:x, :y, :zorder, :text, :fitt], :debug => true))
  end
  
  def reload
    @level.image = Image.new($window, "media/#{@image_name}")
    viewport.game_area = [0,0,@level.width,@level.height]
  end
  
  def update
    super
    
    # viewport.center_around(@player)    
    viewport.x = @player.x - $window.width / 2
    viewport.y = @player.y - $window.height + 100
    
    # if player is at a pixel not black   
    if terrain_collision_with(@player)
      @player.land
    end    
    
    $window.caption = "Unexpected Outcome. ARROWS: move. SPACE: action. LD#19-entry by ippa - http://ippa.se/gaming. FPS: #{$window.fps}"
    ##$window.caption = "Unexpected Outcome. LD#19 Entry by ippa - http://ippa.se/gaming ). [#{@player.x}#/#{@player.y}] FPS: #{$window.fps}"
  end
  
  def terrain_collision_with(game_object)
    
    x = (game_object.x)/$window.factor
    start_y = ((game_object.y-game_object.height)/$window.factor).to_i
    stop_y = ( (game_object.y-1) /$window.factor).to_i
        
    [-2,-1,0,1,2].each do |xoffset|
      x2, y2, color = @level.image.line x+xoffset, start_y, x+xoffset, stop_y, :trace => { :while_color => Color::WHITE }, :color_mode => :gosu
      return true if color
    end
		
    return false
  end
  
end

class Level1 < Level
  def setup
    @song = Song["the_lab.ogg"].play(true)
    @player = Player.create(:x => 40*$window.factor, :y => 240*$window.factor)
    Text.create("ARROW-KEYS to move. SPACE for action.", :x => 20*$window.factor, :y => 190*$window.factor, :size => 14, :factor => 2)
  end
end

class Level2 < Level
  def setup
    Song["outside.ogg"].play(true)
    @player = Player.create(:x => 15*$window.factor, :y => 675*$window.factor)
  end
  
  def update
    super
    if viewport.outside?(@player)
      switch_game_state(Level3)
    end
  end
end

class Level3 < Level
  def setup
    Song["cave.ogg"].play(true)
    @player = Player.create(:x => 22*$window.factor, :y => 23*$window.factor)
    # @player = Player.create(:x => 770*$window.factor, :y => 470*$window.factor)
  end
end

class End < GameState
  trait :timer
  def setup
    Song.current_song.stop if Song.current_song    
    after(15000) { $window.switch_game_state(Outro) }
    self.input = {[:esc, :space] => Outro}
    GameObject.create(:image => "end.png", :rotation_center => :top_left)
    msg = "- This is universe #6, one of the first we created."
    Text.create(msg, :x => 100, :y => 100, :size => 14, :factor => 2, :color => Color::BLACK, :zorder => 100)
  end
end

class Outro < GameState
  def setup
    Song["outro.ogg"].play(true)
    self.input = { :esc => :exit }
    
    msg =  "Created for LD#19 (theme discovery) by ippa (http://ippa.se)\n"
    msg += "Made with Chingu (http://ippa.se/chingu)\n"
    msg += "Powered by Ruby & GOSU (http://www.libgosu.org/)\n"
    msg += "Imagemagic with Texplay (https://github.com/banister/texplay)\n"
    msg += "Music/ambience made with Renoise, GFX with Gimp and Paintbrush."
    
    Text.create(msg, :x => 100, :y => 100, :factor => 2, :size => 14, :max_width => 500)
  end
end