class Player < GameObject
  traits :velocity, :timer, :collision_detection
  trait :bounding_box, :debug => false
    
  def setup
    #self.input = [:up, :holding_left, :holding_right, :space, :released_left, :released_right]
    on_input(:up, :up)
    on_input(:holding_left, :holding_left)
    on_input(:holding_right, :holding_right)
    on_input(:space, :space)
    on_input(:released_left, :released_left)
    on_input(:released_right, :released_right)
    
    self.rotation_center = :center_bottom
    
    @speed = 5
    #@anim = Animation.new(:file => "player.bmp", :size => [5,11], :delay => 70)
    @anim = Animation.new(:file => "player2.bmp", :size => [4,10], :delay => 70)
    @anim.frame_names = { :walk => (1..2) }
    @image = @anim.first
    @jumping = false
        
    self.acceleration_y = 0.5
    self.max_velocity = 10
    cache_bounding_box
  end
  
  def up
    return if @jumping
    self.velocity_y = -10
    @jumping = true
  end
  
  def holding_left
    move(-@speed, 0)
    @image = @anim[:walk].next
    @factor_x = -$window.factor
  end

  def holding_right
    move(@speed, 0)
    @image = @anim[:walk].next
    @factor_x = $window.factor
  end
  def released_left; @image = @anim.first; end
  def released_right; @image = @anim.first; end
  
  def move(x,y)
    if x != 0
      d_x = x.abs / x
      x.to_i.abs.times do
        self.x += d_x
        if @parent.terrain_collision_with(self)
          self.x -= d_x
        end
      end
    end

    if y != 0
      d_y = y.abs / y
      y.to_i.abs.times do
        self.y += d_y
        if @parent.terrain_collision_with(self)
          self.y -= d_y
          land
        end
      end
    end
  end
  
  def land
    stop
    @jumping = false
  end
  
  def space
    Player.each_collision(Actor) do |player, actor|
      actor.invoke
    end
  end
  
end
